#include "sworld3.h"
#include "gba.h"
#include "gbalib.h"
#include "stdlib.h"
#include "misc_actors.h"

extern const Sprite sprite_off;
extern const Sprite sprite_clean;

extern u8 empty_start;
extern enemy_t enemy[max_enemies];
extern enemy_shot_t enemy_shot[max_enemy_shots];
extern player_t player;

extern u8 num_enemies;
extern u8 available_enemies;

extern u8 num_enemy_shots;

u8 enemy_turn (u8 c);
u8 beheomoth_turn (u8 e);

void create_enemy ( void );

// this will be called once per vsync from the main loop
// this is where we decide if we need more enemies etc. 
// anything that isn't done by an individual enemy
void enemy_housekeeping (void)
{
  if (num_enemies < available_enemies ) // max_enemies)
    create_enemy ();
}

void create_beheomoth (u8 c,u16 pos)
{
  enemy[c].last_fire = pos&0xFF;
     
  pos = pos % 330;
  
  if (pos < 100) // alien from top
  {
    enemy[c].x = ((160+pos) << 4);
    enemy[c].y = ((-32)<<4);
    enemy[c].dx = -3 + (pos&3);
    enemy[c].dy = 2;
  }
  else if (pos < 230) // alien from side
  {
    pos = pos - 100;
    enemy[c].x = ((260) << 4);
    enemy[c].y = (pos << 4);
    enemy[c].dx = -(3 + (pos&3));
    enemy[c].dy = 0;
  }
  else // alien from bottom
  {
    pos = pos -230;
    enemy[c].x = ((160+pos) << 4);
    enemy[c].y = (160<<4);
    enemy[c].dx = -(3 + (pos&3));
    enemy[c].dy = -3;
  }

  enemy[c].turn = beheomoth_turn;
  enemy[c].dead = 0;
  enemy[c].size_x = 512;
  enemy[c].size_y = 512;
  enemy[c].sprite = sprite_clean;
  sprite_size (&enemy[c].sprite,2);
  sprite_shape (&enemy[c].sprite,0);
  sprite_tile (&enemy[c].sprite,14);
  sprite_move (&enemy[c].sprite, enemy[c].x>>4, enemy[c].y>>4);
  enemy[c].hit_points = 75;
  enemy[c].score = 150;
 
}

void create_enemy ( void )
{
  u8 c = num_enemies;
  u16 pos;

  if (c < max_enemies)
  {
    num_enemies ++;
    pos = rand();
    if ((pos &31) == 31)
    {
      create_beheomoth (c,pos);
      return;
    }

    enemy[c].last_fire = pos&0xFF;
      
    pos = pos % 330;
  
    if (pos < 100) // alien from top
    {
      enemy[c].x = ((160+pos) << 4);
      enemy[c].y = ((-15)<<4);
      enemy[c].dx = -20 + (pos&15);
      enemy[c].dy = 15;
    }
    else if (pos < 230) // alien from side
    {
      pos = pos - 100;
      enemy[c].x = ((260) << 4);
      enemy[c].y = (pos << 4);
      enemy[c].dx = -(20 + (pos&15));
      enemy[c].dy = 0;
    }
    else // alien from bottom
    {
      pos = pos -230;
      enemy[c].x = ((160+pos) << 4);
      enemy[c].y = (160<<4);
      enemy[c].dx = -(20 + (pos&15));
      enemy[c].dy = -15;
    }

    enemy[c].turn = enemy_turn;
    enemy[c].dead = 0;
    enemy[c].size_x = 256;
    enemy[c].size_y = 256;

    enemy[c].sprite = sprite_clean;
    sprite_size (&enemy[c].sprite,1);
    sprite_shape (&enemy[c].sprite,0);
    sprite_move (&enemy[c].sprite, enemy[c].x>>4, enemy[c].y>>4);
    pos = pos & 16;
    if (pos == 0)
    {
      sprite_tile (&enemy[c].sprite,74);
      enemy[c].hit_points = 2;
      enemy[c].score = 3;
    }
    else
    {
      sprite_tile (&enemy[c].sprite,76);
      enemy[c].hit_points = 1;
      enemy[c].score = 1;
    }
  }
}

void enemy_shot_kill (u8 shot_num)
{
  u8 replace = num_enemy_shots -1;
 
  if (shot_num < replace)
  {
    enemy_shot[shot_num] = enemy_shot[replace];
  }

  enemy_shot[replace].sprite = sprite_off;
  num_enemy_shots --;

}

u8 enemy_shot_turn (u8 shot_num)
{
  enemy_shot[shot_num].x += enemy_shot[shot_num].dx;
  enemy_shot[shot_num].y += enemy_shot[shot_num].dy;

  // check if the bullet should be turned off
  if ( (enemy_shot[shot_num].x > (260 << 4)) ||(enemy_shot[shot_num].x < -(8<<4)) 
     || (enemy_shot[shot_num].y > (160 << 4)) || (enemy_shot[shot_num].y < -(8<<4)) )
  {
    enemy_shot_kill (shot_num);
    sprite_move (&enemy_shot[shot_num].sprite, enemy_shot[shot_num].x>>4, enemy_shot[shot_num].y>>4);
    return 0;
  }
  
  sprite_move (&enemy_shot[shot_num].sprite, enemy_shot[shot_num].x>>4, enemy_shot[shot_num].y>>4);
  return 1;
}

void create_enemy_fire (u16 x, u16 y, s16 dx, s16 dy)
{
  u8 c = num_enemy_shots;
  if (num_enemy_shots < max_enemy_shots)
  {
    enemy_shot[c].x = x;
    enemy_shot[c].y = y;
    enemy_shot[c].dx = dx;
    enemy_shot[c].dy = dy;
    enemy_shot[c].power = 1;
    enemy_shot[c].sprite = sprite_clean;
    sprite_size(&enemy_shot[c].sprite,0);
    sprite_shape(&enemy_shot[c].sprite,0);
    sprite_tile(&enemy_shot[c].sprite,37);

    num_enemy_shots++;
  }

}

void enemy_fire (u8 e)
{
  s16 dx;
  s16 dy;

  if (enemy[e].x < player.x) 
    dx = 16;
  else if (enemy[e].x > player.x + 100)
    dx = -16;
  else 
    dx = 0;

  if (enemy[e].y < player.y) 
    dy = 16;
  else if (enemy[e].y > player.y + 50)
    dy = -16;
  else 
    dy = 0;

  create_enemy_fire (enemy[e].x,enemy[e].y, dx, dy);
  enemy[e].last_fire = rand()&0xFF;    
}

void enemy_off_screen (u8 enemy_num)
{
  u8 last_sprite = num_enemies+enemy_sprite_start-1;  
  u8 replace = num_enemies -1;
  
  if (enemy_num < replace)
  {
    enemy[enemy_num] = enemy[replace];
  }

  enemy[last_sprite].sprite = sprite_off;
  num_enemies --;

}


u8 beheomoth_turn (u8 e)
{

  if (enemy[e].dead == 1)
  {
    create_misc_explosion (enemy[e].x, enemy[e].y);
    create_misc_explosion (enemy[e].x+256, enemy[e].y);
    create_misc_explosion (enemy[e].x+256, enemy[e].y+256);
    create_misc_explosion (enemy[e].x, enemy[e].y+256);
    create_misc_explosion (enemy[e].x+128, enemy[e].y+128);
    enemy_off_screen (e);
    return 0;
  }

  enemy[e].x += enemy[e].dx;
  enemy[e].y += enemy[e].dy;
  if ( (enemy[e].x > (260 << 4)) ||(enemy[e].x < -(enemy[e].size_x)) 
     || (enemy[e].y > (160 << 4)) || (enemy[e].y < -(enemy[e].size_y)) )
  {
    enemy_off_screen (e);
    sprite_move (&enemy[e].sprite, enemy[e].x>>4, enemy[e].y>>4);
    return 0;
  }

  sprite_move (&enemy[e].sprite, enemy[e].x>>4, enemy[e].y>>4);

  // the beheomoth doesn't fire

  return 1;
}

u8 enemy_turn (u8 e)
{

  if (enemy[e].dead == 1)
  {
    create_misc_explosion (enemy[e].x, enemy[e].y);
    enemy_off_screen (e);
    return 0;
  }

  enemy[e].x += enemy[e].dx;
  enemy[e].y += enemy[e].dy;
  if ( (enemy[e].x > (260 << 4)) ||(enemy[e].x < -(enemy[e].size_x)) 
     || (enemy[e].y > (160 << 4)) || (enemy[e].y < -(enemy[e].size_y)) )
  {
    enemy_off_screen (e);
    sprite_move (&enemy[e].sprite, enemy[e].x>>4, enemy[e].y>>4);
    return 0;
  }

  sprite_move (&enemy[e].sprite, enemy[e].x>>4, enemy[e].y>>4);

  if (enemy[e].last_fire > 0)
    enemy[e].last_fire --;
  else
    enemy_fire(e);

  return 1;
}

